@@ -88,8 +88,13 @@ typedef struct {
   gchar *name;
   gchar *handler;
   gchar *object;
+  gchar *state;
+  gchar *target;
 
   GConnectFlags flags;
+
+  guint is_handler : 1;
+  guint warp_to    : 1;
 } SignalInfo;
 
 void property_info_free (gpointer data);
