@@ -58,8 +58,7 @@
 #include <X11/extensions/XInput2.h>
 #endif
 
-#include "cogl/cogl.h"
-#include "cogl/cogl-internal.h"
+#include <cogl/cogl.h>
 
 #include "clutter-backend.h"
 #include "clutter-debug.h"
@@ -132,16 +131,16 @@ cogl_xlib_filter (XEvent       *xevent,
                   gpointer      data)
 {
   ClutterX11FilterReturn retval;
-  CoglXlibFilterReturn ret;
+  CoglFilterReturn ret;
 
-  ret = _cogl_xlib_handle_event (xevent);
+  ret = cogl_xlib_handle_event (xevent);
   switch (ret)
     {
-    case COGL_XLIB_FILTER_REMOVE:
+    case COGL_FILTER_REMOVE:
       retval = CLUTTER_X11_FILTER_REMOVE;
       break;
 
-    case COGL_XLIB_FILTER_CONTINUE:
+    case COGL_FILTER_CONTINUE:
     default:
       retval = CLUTTER_X11_FILTER_CONTINUE;
       break;
@@ -390,7 +389,7 @@ _clutter_backend_x11_post_parse (ClutterBackend  *backend,
 
       /* Cogl needs to know the Xlib display connection for
          CoglTexturePixmapX11 */
-      _cogl_xlib_set_display (backend_x11->xdpy);
+      cogl_xlib_set_display (backend_x11->xdpy);
 
       /* add event filter for Cogl events */
       clutter_x11_add_filter (cogl_xlib_filter, NULL);
